"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCompareScreenshotCommand = void 0;
/// <reference types="cypress" />
const constants_1 = require("./constants");
function addCompareScreenshotCommand(defaultScreenshotOptions = {
    pluginOptions: {
        customSnapshotsDir: "cypress/snapshots"
    }
}) {
    Cypress.Commands.add('compareScreenshot', { prevSubject: 'optional' }, (subject, options = {}) => {
        const specRelativPath = Cypress.spec.relative.replace(/^cypress\/(e2e|integrations)\//, "");
        const screenshotTitle = Cypress.currentTest.titlePath.join(" -- ");
        const screenshotOptions = {
            ...defaultScreenshotOptions.screenshotOptions,
            ...options.screenshotOptions,
        };
        const compareOptions = {
            ...defaultScreenshotOptions.compareOptions,
            ...options.compareOptions,
        };
        const pluginOptions = {
            ...defaultScreenshotOptions.pluginOptions,
            ...options.pluginOptions,
        };
        const expectedScreenshotPath = `${pluginOptions.customSnapshotsDir}/${specRelativPath}/${screenshotTitle}${constants_1.EXPECTED_SUFFIX}.png`;
        const actualScreenshotPath = `${pluginOptions.customSnapshotsDir}/${specRelativPath}/${screenshotTitle}${constants_1.ACTUAL_SUFFIX}.png`;
        const diffScreenshotPath = `${pluginOptions.customSnapshotsDir}/${specRelativPath}/${screenshotTitle}${constants_1.DIFF_SUFFIX}.png`;
        let screenshotPath;
        (subject ? cy.wrap(subject) : cy)
            .screenshot({
            ...screenshotOptions,
            onAfterScreenshot($el, props) {
                screenshotPath = props.path;
            },
        })
            .task(constants_1.TASK.fileExists, expectedScreenshotPath)
            .then((fileExists) => {
            if (pluginOptions.updateSnapshots) {
                cy.task(constants_1.TASK.copyFile, {
                    from: screenshotPath,
                    to: expectedScreenshotPath,
                });
            }
            else if (!fileExists) {
                if (pluginOptions.failOnExpectedMissing &&
                    !pluginOptions.updateSnapshots) {
                    throw new Error("Expected image is missing");
                }
                cy.task(constants_1.TASK.copyFile, {
                    from: screenshotPath,
                    to: expectedScreenshotPath,
                }).then((copyResult) => {
                    if (copyResult !== true) {
                        throw new Error(copyResult);
                    }
                });
            }
            else {
                cy.task(constants_1.TASK.copyFile, {
                    from: screenshotPath,
                    to: actualScreenshotPath,
                }).then(() => {
                    cy.task(constants_1.TASK.compare, {
                        expectedScreenshotPath,
                        actualScreenshotPath,
                        diffScreenshotPath,
                        compareOptions,
                    }).then((result) => {
                        if (result !== true) {
                            throw new Error(result);
                        }
                    });
                });
            }
        });
    });
}
exports.addCompareScreenshotCommand = addCompareScreenshotCommand;
