"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCompareScreenshotPlugin = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const odiff_bin_1 = require("odiff-bin");
const constants_1 = require("./constants");
async function fileExists(filePath) {
    const absoluteFilePath = path_1.default.join(process.cwd(), filePath);
    return fs_1.default.existsSync(absoluteFilePath);
}
async function copyFile(args) {
    try {
        const absoluteToPath = path_1.default.join(process.cwd(), args.to);
        fs_1.default.mkdirSync(path_1.default.dirname(absoluteToPath), { recursive: true });
        fs_1.default.renameSync(args.from, absoluteToPath);
    }
    catch (error) {
        return error.message;
    }
    return true;
}
async function compareScreenshotsTask(args) {
    // const shouldUpdateSnapshots = args.updateSnapshots;
    const expectedAbsolutePath = path_1.default.join(process.cwd(), args.expectedScreenshotPath);
    const actualAbsolutePath = path_1.default.join(process.cwd(), args.actualScreenshotPath);
    const diffAbsolutePath = path_1.default.join(process.cwd(), args.diffScreenshotPath);
    const compareResult = await (0, odiff_bin_1.compare)(actualAbsolutePath, expectedAbsolutePath, diffAbsolutePath, args.compareOptions);
    if (compareResult.match) {
        return true;
    }
    switch (compareResult.reason) {
        case "layout-diff":
            return "Screenshots does not have the same layout";
        case "pixel-diff":
            return `Screenshots does not match. ${compareResult.diffCount} pixels difference (${compareResult.diffPercentage}%)`;
        case "file-not-exists":
            return `File was not found ${compareResult.file}`;
    }
    return "Unknown Error: match failed without reason";
}
function addCompareScreenshotPlugin(on, config) {
    on('task', {
        [constants_1.TASK.compare]: async (args) => compareScreenshotsTask(args),
        [constants_1.TASK.fileExists]: async (args) => fileExists(args),
        [constants_1.TASK.copyFile]: async (args) => copyFile(args),
    });
}
exports.addCompareScreenshotPlugin = addCompareScreenshotPlugin;
